"""Type definitions for pymemuc"""

from typing import Literal, TypedDict


class VMInfo(TypedDict):
    """A TypedDict for VM info"""

    index: int
    """ VM index """
    title: str
    """VM title"""
    top_level: str
    """VM top level window"""
    running: bool
    """VM is running"""
    pid: int
    """VM process ID"""
    disk_usage: int
    """VM disk usage in bytes"""


ConfigKeys = Literal[
    "name",
    "cpus",
    "cpucap",
    "memory",
    "is_full_screen",
    "is_hide_toolbar",
    "turbo_mode",
    "graphics_render_mode",
    "enable_su",
    "enable_audio",
    "fps",
    "vkeyboard_mode",
    "sync_time",
    "phone_layout",
    "start_window_mode",
    "win_x",
    "win_y",
    "win_scaling_percent2",
    "is_customed_resolution",
    "resolution_width",
    "resolution_height",
    "vbox_dpi",
    "linenum",
    "imei",
    "imsi",
    "simserial",
    "microvirt_vm_brand",
    "microvirt_vm_model",
    "microvirt_vm_manufacturer",
    "selected_map",
    "latitude",
    "longitude",
    "picturepath",
    "musicpath",
    "moviepath",
    "downloadpath",
    "macaddress",
    "cache_mode",
    "geometry",
    "custom_resolution",
    "disable_resize",
    "ssid",
]



"""This module contains decorators for functions in pymemuc."""

from functools import wraps
from typing import TYPE_CHECKING, Any, Callable

from ._constants import RETRIES
from .exceptions import PyMemucError, PyMemucTimeoutExpired

if TYPE_CHECKING:
    from pymemuc import PyMemuc


def retryable(func: Callable[..., Any]):
    """Decorator to retry a function if it raises an exception.
    The number of retries is defined in pymemuc._constants.RETRIES
    After the last retry, the exception is raised.
    """

    @wraps(func)
    def wrapper(self: "PyMemuc", *args: Any, **kwargs: Any):
        fin_err = None  # track the last error
        for i in range(RETRIES):
            try:
                return func(self, *args, **kwargs)
            except (PyMemucError, PyMemucTimeoutExpired) as err:
                fin_err = err  # update the last error
                self.logger.debug(f"pymemuc._decorators._retryable: {err}")
                r_left = RETRIES - i - 1
                if r_left > 0:
                    self.logger.debug(
                        f"\tretrying {r_left} more time{'s' if r_left > 1 else ''}..."
                    )
        raise PyMemucError(f"Max retries ({RETRIES}) exceeded") from fin_err

    return wrapper
